/* ======================================================== */
/* global require */
const { task, src, dest, watch, series, parallel } = require("gulp"); // Собственно gulp

const clean = require("gulp-clean"); // удаление файлов и каталогов

const include = require("gulp-file-include");
const version = require("gulp-version-number"); // версионность файлов

const postcss = require("gulp-postcss"); // Пост-обработка CSS
const postcssPresetEnv = require("postcss-preset-env"); // CSS Next
const atImport = require("postcss-import"); // парсинг и включение импорта CSS

const sourcemaps = require("gulp-sourcemaps"); // карты исходного кода

const webpack = require("webpack-stream");
const compiler = require("webpack");

const imagemin = require("gulp-image"); // оптимизация картинок
const cache = require("gulp-cache");

const browsersync = require("browser-sync").create(); // обновление налету

/* ========================================================== */

/**
 * Настройки CSS
 */
const postcssOptions = [
	atImport(),
	postcssPresetEnv({
		stage: 0,
		autoprefixer: {
			grid: true,
			browsers: ["last 4 versions", "ie >= 11", "iOS >= 9"]
		}
	})
];

/* ================================================================= */

/**
 * CLEAN
 * Полная очистка
 */
const Clean = cb => {
	cb();
	return src(["./dev", "./prod"], { read: false, allowEmpty: true }).pipe(
		clean()
	);
};
task("CLEAN", Clean);

/**
 * CSS
 */
const devCss = cb => {
	cb();
	return src("./src/assets/css/*.css")
		.pipe(sourcemaps.init())
		.pipe(postcss(postcssOptions))
		.pipe(sourcemaps.write("."))
		.pipe(dest("../prodalim/frontend/web/assets/css"))
		.pipe(browsersync.stream());
};

// Собираем CSS
const prodCss = cb => {
	cb();
	return src("./src/assets/css/*.css")
		.pipe(postcss(postcssOptions))
		.pipe(dest("./prod/assets/css"));
};

// ================== JS ============================
// Компилируем и собираем все js файлы проекта
const devJs = cb => {
	cb();
	return (
		src("./src/assets/js/*.js")
			.pipe(
				webpack(require("./webpack.dev.js"), compiler, function(
					err,
					stats
				) {
					// console.log( err, stats );
				})
			)
			// .pipe( dest( './dev/assets/js' ) )
			.pipe(dest("../prodalim/frontend/web/assets/js"))
			.pipe(browsersync.stream())
	);
};

// Обрабатываем js файлы библиотек и проекта
const prodJs = cb => {
	cb();
	return src("./src/assets/js/*.js")
		.pipe(
			webpack(require("./webpack.prod.js"), compiler, function(
				err,
				stats
			) {
				// console.log( err, stats );
			})
		)
		.pipe(dest("./prod/assets/js"));
};

// ================== IMG ============================
// Оптимизация картинок для dev
const devImgCommon = cb => {
	cb();
	return src([
		"!./src/favicon/**/*",
		"!./src/assets/**/*",
		"!./src/components/**/*",
		"./src/**/*.png",
		"./src/**/*.jpg",
		"./src/**/*.jpeg",
		"./src/**/*.gif",
		"./src/**/*.svg"
	])
		.pipe(cache(imagemin()))
		.pipe(dest("./dev"));
};

const devImgComponents = cb => {
	cb();
	return src([
		"./src/assets/img/**/*",
		"./src/components/**/*.png",
		"./src/components/**/*.jpg",
		"./src/components/**/*.jpeg",
		"./src/components/**/*.gif",
		"./src/components/**/*.svg"
	])
		.pipe(cache(imagemin()))
		.pipe(dest("./dev/assets/img"));
};

// Оптимизация картинок для prod
const prodImgCommon = cb => {
	cb();
	return src([
		"src/**/*.png",
		"src/**/*.jpg",
		"src/**/*.jpeg",
		"src/**/*.gif",
		"src/**/*.svg",
		"!src/favicon/",
		"!src/assets/",
		"!src/components/"
	])
		.pipe(cache(imagemin()))
		.pipe(dest("prod"));
};

const prodImgComponents = cb => {
	cb();
	return src([
		"src/assets/img/**/*",
		"src/components/**/*.png",
		"src/components/**/*.jpg",
		"src/components/**/*.jpeg",
		"src/components/**/*.gif",
		"src/components/**/*.svg"
	])
		.pipe(cache(imagemin()))
		.pipe(dest("prod/assets/img"));
};

// ================== HTML ============================
// HTML для dev
const devHtml = () => {
	return src([
		"src/**/*.html",
		"!src/assets/*.html",
		"!src/components/**/*.html"
	])
		.pipe(
			include({
				prefix: "@@",
				basepath: "@file",
				context: {
					env: "DEV",
					path: "/assets"
				}
			})
		)
		.pipe(
			version({
				value: "%MDS%",
				append: {
					key: "v",
					to: ["css", "js", ["image", "%TS%"]]
				}
			})
		)
		.pipe(dest("./dev"));
};

// HTML для prod
const prodHtml = cb => {
	cb();
	return src([
		"src/**/*.html",
		"!src/assets/*.html",
		"!src/components/**/*.html"
	])
		.pipe(
			include({
				prefix: "@@",
				basepath: "@file",
				context: {
					env: "PROD",
					path: "/assets"
				}
			})
		)
		.pipe(
			version({
				value: "%MDS%",
				append: {
					key: "v",
					to: ["css", "js", ["image", "%TS%"]]
				}
			})
		)
		.pipe(dest("./prod"));
};

// ================== DEV ============================
// Слежение за dev
function browserSync() {
	browsersync.init({
		/* server: {
			baseDir: '../prodalim/frontend/web/'
		}, */
		proxy: "prodalim.local"
		// port: 3000
	});
}

function browserSyncReload(done) {
	browsersync.reload();
	done();
}

task(
	"DEV",
	series(devImgCommon, devImgComponents, devCss, devJs, devHtml, () => {
		watch(["./src/assets/css/*.css", "./src/components/**/*.css"], devCss);

		watch(["./src/assets/js/*.js", "./src/components/**/*.js"], devJs);

		watch("./src/**/*.html", series(devHtml, browserSyncReload));

		watch(
			[
				"./src/assets/img/**/*",
				"./src/components/**/*.gif",
				"./src/components/**/*.jpg",
				"./src/components/**/*.jpeg",
				"./src/components/**/*.png",
				"./src/components/**/*.svg"
			],
			parallel(devImgCommon, devImgComponents)
		);

		browserSync();
	})
);

// ================== PROD ============================
// Публикация на prod
task(
	"PROD",
	series(prodImgCommon, prodImgComponents, prodCss, prodJs, prodHtml)
);
